#
# spec file for package gasnet
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define _buildshell /bin/bash

Name:           gasnet
Version:        1.32.0
Release:        bp156.3.15
%define libver	1_32_0
Summary:        A Communication Layer for GAS Languages
License:        PostgreSQL
Group:          Productivity/Networking/Other
URL:            https://gasnet.lbl.gov
Source0:        https://gasnet.lbl.gov/download/GASNet-%{version}.tar.gz
Patch0:         gasnet-date-time.patch
# PATCH-FIX-OPENSUSE -- have constant BUILD_ID to fix build-compare
Patch2:         gasnet-build-id.patch
# PATCH-FIX-OPENSUSE https://bitbucket.org/berkeleylab/gasnet/pull-requests/253/allow-to-not-store-build-date-user-and/diff
Patch3:         gasnet-build-hostname.patch
Patch4:         gasnet-fix-multiple-definitions.patch
BuildRequires:  automake
BuildRequires:  fdupes
BuildRequires:  gcc-c++
BuildRequires:  libibverbs-devel
%ifarch x86_64 %{ix86}
BuildRequires:  libfabric-devel
%endif
%ifarch x86_64
BuildRequires:  libpsm2-devel
%endif
BuildRequires:  libtool
BuildRequires:  openmpi-macros-devel

BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
GASNet (Global Address Space Networking) is a language-independent,
low-level networking layer that provides network-independent
communication primitives tailored for implementing parallel global
address space SPMD languages such as UPC, Titanium, and Co-Array
Fortran.

%package devel
Summary:        Development files for GASNet
Group:          Development/Libraries/C and C++
Requires:       %{name} = %{version}
Requires:       lib%{name}-%{libver}

%description devel
GASNet is a language-independent, low-level networking layer that provides
network-independent communication primitives tailored for
implementing parallel global address space SPMD languages
such as UPC, Titanium, and Co-Array Fortran.

The interface is primarily intended as a compilation target and for
use by runtime library writers (as opposed to end users).

Development package for GASNet. Including header files and libraries.

%package -n libgasnet-%{libver}
Summary:        Runtime libraries for GASNet
Group:          System/Libraries
%openmpi_requires

%description -n libgasnet-%{libver}
GASNet is a language-independent, low-level networking layer that provides
network-independent communication primitives tailored for
implementing parallel global address space SPMD languages
such as UPC, Titanium, and Co-Array Fortran.

This package contains the libraries for GASNet.

%package doc
Summary:        Documentation for GASNet
Group:          Documentation/Other
BuildArch:      noarch

%description doc
GASNet is a language-independent, low-level networking layer that provides
network-independent communication primitives tailored for
implementing parallel global address space SPMD languages
such as UPC, Titanium, and Co-Array Fortran.

This package contains the documentation for GASNet.

%prep
%setup -q -n GASNet-%{version}
%patch0
%patch2 -p1
%patch3 -p1
%patch4

# Avoid unnecessary rebuilds of the package
FAKE_BUILDDATE=$(LC_ALL=C date -u -r %{_sourcedir}/%{name}.changes '+%%b %%e %%Y')
FAKE_BUILDTIME=$(LC_ALL=C date -u -r %{_sourcedir}/%{name}.changes '+%%H:%%M:%%S')
sed -i -e "s/__DATE__/\"$FAKE_BUILDDATE\"/" -e "s/__TIME__/\"$FAKE_BUILDTIME\"/" gasnet_tools.c gasnet_trace.c tests/test.h

%build
%setup_openmpi
%configure --enable-udp --enable-mpi --enable-par --enable-ibv --disable-aligned-segments  --enable-segment-fast --with-segment-mmap-max=4GB --disable-debug --enable-force-posix-realtime --enable-smp \
%ifarch x86_64 %{ix86}
    --enable-ofi \
%endif
%ifarch x86_64
    --enable-psm \
%endif
%ifarch s390 s390x
  --enable-force-compiler-atomicops \
%endif

 CC="gcc -fPIC" CXX="g++ -fPIC"

make %{?_smp_mflags} MANUAL_CFLAGS="%optflags -fPIC" MANUAL_MPICFLAGS="%optflags -fPIC" MANUAL_CXXFLAGS="%optflags -fPIC"

%check
%setup_openmpi
make %{?_smp_mflags} check MANUAL_CFLAGS="%optflags -fPIC" MANUAL_MPICFLAGS="%optflags -fPIC" MANUAL_CXXFLAGS="%optflags -fPIC"

%install
%setup_openmpi
make %{?_smp_mflags} DESTDIR=%{buildroot} install
%fdupes %{buildroot}/%{_prefix}
chmod +x %{buildroot}/%{_bindir}/*.pl

#Upstream doesn't want to support shared libs: https://bitbucket.org/berkeleylab/gasnet/pull-requests/36
#mind the order for link deps, libgasnet-smp-par first then libam* then the rest
for l in %{buildroot}/%{_libdir}/lib{gasnet-smp-par,am*,*}.a; do \
    [[ -f $l ]] || continue; \
    soname=`basename $l .a`; \
    libdir=`dirname $l`; \
	linker=`which g++`
    libs= ; \
    [[ ${soname} = libgasnet-*-par* ]] && libs+=" -lpthread"; \
    [[ ${soname} = libgasnet_*-par* ]] && libs+=" -lpthread"; \
    [[ ${soname} = libgasnet-psm-seq ]] && libs+=" -lpthread"; \
    [[ ${soname} = libamudp ]] && libs+=" -L${libdir} -lgasnet-smp-par"; \
    [[ ${soname} = libammpi ]] && libs+=" $(mpicc --showme:link) -L${libdir} -lgasnet-smp-par"; \
    [[ ${soname} = libgasnet-udp-* ]] && libs+=" -L${libdir} -lamudp"; \
    [[ ${soname} = libgasnet-mpi-* ]] && libs+=" -L${libdir} -lammpi"; \
    [[ ${soname} = libgasnet-ibv-* ]] && libs+=" -L${libdir} -libverbs -lpthread" &&
          linker=`which mpic++`; \
    [[ ${soname} = libgasnet-psm-* ]] && libs+=" -L${libdir} -lpsm2" &&
	   linker=`which mpic++`; \
    [[ ${soname} = libgasnet-ofi-* ]] && libs+=" -L${libdir} -lfabric" &&
	   linker=`which mpic++`; \
    [[ ${soname} = libgasnet-*-* ]] && libs+=" -lrt"; \
    (${linker} -shared -Wl,-soname=${soname}-%{version}.so \
        -Wl,--as-needed -Wl,-z,defs -Wl,--rpath-link=. \
        -Wl,--whole-archive ${l} -Wl,--no-whole-archive \
        ${libs} -o ${libdir}/${soname}-%{version}.so && \
    ln -s ${soname}-%{version}.so ${libdir}/${soname}.so && \
    rm ${l}) || exit 1 ; \
done

%post -n libgasnet-%{libver} -p /sbin/ldconfig
%postun -n libgasnet-%{libver} -p /sbin/ldconfig

%files
%defattr(-,root,root)
%{_bindir}/*

%files -n libgasnet-%{libver}
%defattr(-,root,root)
%{_libdir}/lib*-%{version}.so

%files doc
%defattr(-,root,root)
%{_datadir}/doc/GASNet

%files devel
%defattr(-,root,root)
%{_includedir}/*
%{_libdir}/lib*[a-z].so
%{_libdir}/pkgconfig/*.pc
%{_libdir}/valgrind

%changelog
* Sun Jun  7 2020 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Add gasnet-fix-multiple-definitions.patch to fix compatibility with GCC10
* Fri Oct  4 2019 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Always use the default openmpi
* Thu Sep 26 2019 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Update to GASNet 1.32.0
  - Libfabric (ofi-conduit)
  - Fix bug 3682: incorrect behavior for non-bulk puts larger than a page
  - Compatibility fixes for libfabric version 1.6
  - InfiniBand Verbs (ibv-conduit)
  - GASNET_DISABLE_MUNMAP now defaults to enabled for ibv-conduit.
  - Now use HCA's maximum supported MTU by default, resulting in nearly a 2x
    increase in peak Get bandwidth on modern HCAs.
  - Fix bug 3339: allow ibv-conduit to operate on Omni-Path hardware.
    ofi-conduit and psm-conduit remain the recommended conduits for Omni-Path.
  - Portable UDP (udp-conduit)
  - Reduce polling cost on single-supernode runs. smp-conduit remains the
    recommended conduit for use on single-supernode shared-memory systems.
  - Portable MPI (mpi-conduit)
  - Reduce polling cost on single-supernode runs. smp-conduit remains the
    recommended conduit for use on single-supernode shared-memory systems.
  - GASNet tools (spec v1.12)
  - Numerous improvements to backtrace support, including fixing the
    following bugs:
  - Bug 3624: Improve backtrace fail-over behavior
  - Bug 3625: call prctl(PR_SET_PTRACER,...) before backtrace on Linux
  - Bug 3626: prioritize lldb on Mac OS X
  - Bug 3627: unblock the unfreeze signal prior to backtrace
  - Bug 3637: Fix EXECINFO backtrace for *BSD
  - Enable GDB backtrace on Cray systems, when available
  - Bug 3664: GASNETT_PREDICT_{TRUE,FALSE} result value semantics change slightly
  - Add gasnett_unreachable()
  - GASNETT_PLEASE_INLINE has been removed. Recommended replacement is `inline`
  - Improve behavior when installed public headers are passed to a compiler
    that was not probed at configure time.
  - General and Misc.
  - GASNET_MAX_SEGSIZE syntax has been expanded and now defaults to 85%% of physical
    memory, split evenly between co-located processes on a host.
    Configure option to control this default renamed to --with-max-segsize=
  - GASNET_VIS_{AMPIPE,REMOTECONTIG} is now enabled by default on most conduits
  - GASNET_VIS_MAXCHUNK default value adjusted on many conduits to improve perf
  - New knobs GASNET_VIS_{PUT,GET}_MAXCHUNK for tuning put/get VIS chunk size
  - Fix GASNET_NULL_ARGV_OK for mpi-conduit and mpi-spawner
  - Additional debug checks now detect prohibited overlap in loopback RMA and AMLong.
  - The extended-ref source directory has been re-arranged
  - Clients using gasnet_coll_* must now explicitly #include <gasnet_coll.h>
  - Events leading up to a crash are now more reliably reported in the trace file.
  - Platform support/portability
  - Support for Microsoft Windows 10 Subsystem for Linux (WSL) has graduated
    out of BETA status
  - Update MALLOC_OPTIONS setting for OpenBSD 6.x
  - Add gasnetrun support for IBM Job Step Manager (jsrun)
  - Add PLATFORM_COMPILER_C(XX)_LANGLVL to portable_platform (v5)
  - Fix bug 3741: misidentification of IBM XL Compilers 13.1.6 and newer (v6)
  - Fix bug 3743: verify correct KNL processor tuning on CNL
  - Fix bug 3679: AMUDP incompatibility with PGI C++ 17+
  - Public headers are less susceptible to malfunctions when compiled with
    un-prefixed tokens redefined by the preprocessor. C/C++ keywords remain
    reserved, as do tokens prepended by one or more underscores or one of the
    reserved gasnet prefixes (see 'GASNet coding standards' in README-devel)
  - Configure and Build
  - Option --enable-strict-prototypes has been removed, see bug 3608.
  - Fix bug 3613: warning on #pragma GCC diagnostic for gcc < 4.6
  - Fix bug 3629: avoid a parallel make bug
  - Add a git hash "watermark" to libraries
  - Warning enable flags are removed from GASNET_C(XX)FLAGS and friends.
    The flags remain available in GASNET_DEVWARN_C(XX)FLAGS for opt-in.
- Drop gasnet-s390-support.patch as it was fixed upstream
- Enable SMP and PSM conduit.
* Mon Jan  7 2019 Bernhard Wiedemann <bwiedemann@suse.com>
- Add gasnet-build-hostname.patch to make build reproducible (boo#1084909)
* Thu Oct  5 2017 nmoreychaisemartin@suse.com
- Force load of MPI environment
- Use OpenMPI2 instead of OpenMPI1 except for ppc64be (unsupported by OpenMPI >= 2)
* Wed Sep  6 2017 nmoreychaisemartin@suse.com
- Force timer to use POSIX RT instead of native, as native is not
  precise enough in VM and causes validation to crash.
* Fri Sep  1 2017 junghans@votca.org
- Version update to 1.30.0:
  * InfiniBand Verbs (ibv-conduit)
  - Improve the handling of GASNET_PHYSMEM_* env vars
  * Libfabric (ofi-conduit)
  - Fix bug 3426: Unbounded recursion within gasnetc_ofi_poll()
  - Significant improvements to AM buffer management and wire header overheads
  - Finer-grained control over AM buffering, see ofi-conduit/README
  - Initialization cleanups for stricter compliance with the OFI specification
  - Support for the Intel(R) True Scale Fabric provider, psm, has been removed.
    The Intel(R) Omni-Path Fabric is still supported through the psm2 provider.
  - We now recommend ofi-conduit for the Intel Omni-Path fabric.
  - The conduit has been validated on libfabric version 1.5.0.
  * Intel Omni-Path PSM2 (psm-conduit)
  - This conduit is now DEPRECATED.
  - Use of ofi-conduit with libfabric's psm2 provider is recommended for users
    of the Intel Omni-Path fabric.
  * GASNet tools
  - Fix bug 3448: x86*/Linux timers once again default to use of the TSC due
    to addition of new logic for correct calibration on newer Intel CPUs.
  - Add gasnett_nsleep()
* Fri Aug 11 2017 bwiedemann@suse.com
- Add gasnet-build-id.patch to fix build-compare (boo#1047218)
* Fri Jun 30 2017 nmoreychaisemartin@suse.com
- Remove BuildRequires for libpsm2-compat
- Build with OFI support on x86
* Fri Jun  9 2017 nmoreychaisemartin@suse.com
- Enable OFI conduit on x86_64
- Disable debug to build GASnet in optimized mode
- Add gasnet-date-time.patch to remove build date/times from binaries
- Add gasnet-s390-support.patch and enable s390/s390x builds
* Thu Jun  8 2017 nmoreychaisemartin@suse.com
- Remove version dependency to libibverbs
* Wed May 31 2017 nmoreychaisemartin@suse.com
- Build gasnet for all arch but s390*, s390x not supported by upstream:
  https://upc-bugs.lbl.gov/bugzilla/show_bug.cgi?id=3523
- PSM2 support is only for x86_64
* Tue May 16 2017 nmoreychaisemartin@suse.com
- Support gasnet over OFI and ibverbs
* Fri Apr  7 2017 pth@suse.de
- Remove conflict marker.
* Fri Mar 31 2017 nmoreychaisemartin@suse.com
- Fix dynamic linking for PSM libraries
* Fri Mar 31 2017 pth@suse.de
- GASnet relies on libpsm2-compat which again only supports x86_64
  so make GASnet do the same.
* Wed Mar 29 2017 pth@suse.de
- Make dependencies on libs now coming from rdma-core versioned.
* Sat Mar 18 2017 junghans@votca.org
- Version update to 1.28.2, lot of changes, see:
  https://gasnet.lbl.gov/dist/ChangeLog
* Mon Feb  6 2017 jengelh@inai.de
- Declare bash as build shell
* Mon Oct 24 2016 junghans@votca.org
- Version update to 1.28.0, lot of changes, see:
  https://gasnet.lbl.gov/dist/ChangeLog
* Mon Oct 17 2016 jengelh@inai.de
- Implement shared library naming guideline
- Trim description
* Tue Sep 20 2016 junghans@votca.org
- added %%check block and related patch from upstream:
  https://bitbucket.org/berkeleylab/gasnet/pull-requests/37
- link shared libs, so that deps are fulfilled
* Mon Sep 12 2016 junghans@votca.org
- Added upstream patch to support overwriting of flags:
  https://bitbucket.org/berkeleylab/gasnet/pull-requests/34
- Build shared libs manually as upstream doesn't want to support them:
  https://bitbucket.org/berkeleylab/gasnet/pull-requests/36
  but we need them for legion package
* Sun Sep  4 2016 junghans@votca.org
- Update to GASNet 1.26.4
  Details: https://gasnet.lbl.gov/dist/ChangeLog
* Sun May 29 2016 junghans@votca.org
- import into Science overlay
* Thu Mar 10 2016 nirmoy.das@suse.com
- Initial package, version: 1.26.0(fate#319250)
